-- Farming Simulator 19 - ChoppedStraw
-- by Nico73 Modding
-- Version 1.0.0.0
-- 16-05-2021
------------------------------------------------

function Combine:processCombineChopperArea(workArea)
	local spec = self.spec_combine

	if not spec.isSwathActive then
		local litersToDrop = spec.workAreaParameters.litersToDrop
		local strawRatio = spec.workAreaParameters.strawRatio
		spec.workAreaParameters.droppedLiters = litersToDrop

		if litersToDrop > 0 and strawRatio > 0 then
			local xs, _, zs = getWorldTranslation(workArea.start)
			local xw, _, zw = getWorldTranslation(workArea.width)
			local xh, _, zh = getWorldTranslation(workArea.height)
			FSDensityMapUtil.updateFertilizerArea(xs, zs, xw, zw, xh, zh, g_currentMission.chopperGroundLayerType)
			FSDensityMapUtil.setGroundTypeLayerArea(xs, zs, xw, zw, xh, zh, g_currentMission.chopperGroundLayerType)
			self:raiseActive()

			spec.workAreaParameters.isChopperEffectEnabled = 500
		end
	end

	return spec.workAreaParameters.lastRealArea, spec.workAreaParameters.lastArea
end

function Cutter:processCutterArea(workArea, dt)
	local spec = self.spec_cutter

	if spec.workAreaParameters.combineVehicle ~= nil then
		local xs, _, zs = getWorldTranslation(workArea.start)
		local xw, _, zw = getWorldTranslation(workArea.width)
		local xh, _, zh = getWorldTranslation(workArea.height)
		local lastRealArea = 0
		local lastThreshedArea = 0
		local lastArea = 0

		for fruitType, state in pairs(spec.workAreaParameters.fruitTypesToUse) do
			if state then
				local realArea, area, sprayFactor, plowFactor, limeFactor, weedFactor, growthState, _, terrainDetailPixelsSum = FSDensityMapUtil.cutFruitArea(fruitType, xs, zs, xw, zw, xh, zh, true, true, spec.allowsForageGrowthState, g_currentMission.chopperGroundLayerType)

				if realArea > 0 then
					if self.isServer then
						if growthState ~= spec.currentGrowthState then
							spec.currentGrowthStateTimer = spec.currentGrowthStateTimer + dt

							if spec.currentGrowthStateTimer > 500 or spec.currentGrowthStateTime + 1000 < g_time then
								spec.currentGrowthState = growthState
								spec.currentGrowthStateTimer = 0
							end
						else
							spec.currentGrowthStateTimer = 0
							spec.currentGrowthStateTime = g_time
						end

						spec.currentInputFruitType = fruitType

						if terrainDetailPixelsSum > 0 then
							spec.currentInputFruitTypeAI = fruitType
						end

						spec.currentInputFillType = g_fruitTypeManager:getFillTypeIndexByFruitTypeIndex(fruitType)
						spec.useWindrow = false
					end

					local multiplier = g_currentMission:getHarvestScaleMultiplier(fruitType, sprayFactor, plowFactor, limeFactor, weedFactor)
					lastRealArea = realArea * multiplier
					lastThreshedArea = realArea
					lastArea = area
					spec.workAreaParameters.lastFruitType = fruitType

					break
				end
			end
		end

		if lastArea > 0 then
			if workArea.chopperAreaIndex ~= nil then
				local workArea = self:getWorkAreaByIndex(workArea.chopperAreaIndex)

				if workArea ~= nil then
					local xs, _, zs = getWorldTranslation(workArea.start)
					local xw, _, zw = getWorldTranslation(workArea.width)
					local xh, _, zh = getWorldTranslation(workArea.height)
					FSDensityMapUtil.updateFertilizerArea(xs, zs, xw, zw, xh, zh, g_currentMission.chopperGroundLayerType)
					FSDensityMapUtil.setGroundTypeLayerArea(xs, zs, xw, zw, xh, zh, g_currentMission.chopperGroundLayerType)
				else
					workArea.chopperAreaIndex = nil

					g_logManager:xmlWarning(self.configFileName, "Invalid chopperAreaIndex '%d' for workArea '%d'!", workArea.chopperAreaIndex, workArea.index)
				end
			end

			spec.isWorking = true
		end

		spec.workAreaParameters.lastRealArea = spec.workAreaParameters.lastRealArea + lastRealArea
		spec.workAreaParameters.lastThreshedArea = spec.workAreaParameters.lastThreshedArea + lastThreshedArea
		spec.workAreaParameters.lastStatsArea = spec.workAreaParameters.lastStatsArea + lastThreshedArea
		spec.workAreaParameters.lastArea = spec.workAreaParameters.lastArea + lastArea
	end

	return spec.workAreaParameters.lastRealArea, spec.workAreaParameters.lastArea
end

function FruitPreparer:processFruitPreparerArea(workArea)
	local spec = self.spec_fruitPreparer
	local workAreaSpec = self.spec_workArea
	local xs, _, zs = getWorldTranslation(workArea.start)
	local xw, _, zw = getWorldTranslation(workArea.width)
	local xh, _, zh = getWorldTranslation(workArea.height)
	local dxs = xs
	local dzs = zs
	local dxw = xw
	local dzw = zw
	local dxh = xh
	local dzh = zh

	if workArea.dropWorkAreaIndex ~= nil then
		local dropArea = workAreaSpec.workAreas[workArea.dropWorkAreaIndex]

		if dropArea ~= nil then
			dxs, _, dzs = getWorldTranslation(dropArea.start)
			dxw, _, dzw = getWorldTranslation(dropArea.width)
			dxh, _, dzh = getWorldTranslation(dropArea.height)
			FSDensityMapUtil.updateFertilizerArea(dxs, dzs, dxw, dzw, dxh, dzh, g_currentMission.chopperGroundLayerType)			
		end
	end

	local area = FSDensityMapUtil.updateFruitPreparerArea(spec.fruitType, xs, zs, xw, zw, xh, zh, dxs, dzs, dxw, dzw, dxh, dzh)

	if area > 0 then
		spec.isWorking = true
	end

	return 0, area
end